package com.digiwin.athena.app.infra.common.utils;

import com.digiwin.athena.app.infra.constant.ParameterConstant;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @description 响应结果转换
 * @throws
 * @time 2023/8/30 13:21
 */
public class ResponseUtil {
    /**
     * 把对象中的 String 类型的null字段，转换为空字符串
     *
     * @param <T> 待转化对象类型
     * @param cls 待转化对象
     * @return 转化好的对象
     */
    public static <T> T noNullStringAttr(T cls) {
        Field[] fields = cls.getClass().getDeclaredFields();
        if (fields == null || fields.length == 0) {
            return cls;
        }
        for (Field field : fields) {
            if (ParameterConstant.STRING.equals(field.getType().getSimpleName())) {
                field.setAccessible(true);
                try {
                    Object value = field.get(cls);
                    if (value == null) {
                        field.set(cls, "");
                    }
                } catch (IllegalArgumentException | IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
        }
        return cls;
    }

    /**
     * 把集合中的所有对象中的String类型的null字段，转换为空字符串
     * 注意：只能转换String类型的字段
     *
     * @param sourceList 待转化的集合
     * @return 转化好的集合
     */
    public static <T> List<T> listNullToString(List<T> sourceList) {
        ArrayList<T> resultList = new ArrayList<>();
        for (T cls : sourceList) {
            Field[] fields = cls.getClass().getDeclaredFields();
            if (fields == null || fields.length == 0) {
                resultList.add(cls);
            }
            for (Field field : fields) {
                if (ParameterConstant.STRING.equals(field.getType().getSimpleName())) {
                    field.setAccessible(true);
                    try {
                        Object value = field.get(cls);
                        if (value == null) {
                            field.set(cls, "");
                        }
                    } catch (IllegalArgumentException | IllegalAccessException e) {
                        e.printStackTrace();
                    }
                }
            }
            resultList.add(cls);
        }
        return resultList;
    }

    /**
     * 注意：只能转换String类型的字段
     *
     * @param sourceList 待转化的集合
     * @return 转化好的集合
     */
    public static List<Map<String, Object>> listMapToLowerCase(List<Map<String, Object>> sourceList) {
        List<Map<String, Object>> response = new ArrayList<>();
        for (Map<String, Object> map : sourceList) {
            Map<String, Object> newMap = new HashMap<>();
            for (Map.Entry<String, Object> entry : map.entrySet()) {
                String key = entry.getKey().toLowerCase();
                Object value = entry.getValue();
                newMap.put(key, value);
            }
            response.add(newMap);
        }
        return response;
    }

}
