package com.digiwin.athena.app.ptc.service.managementdetail;

import com.alibaba.fastjson.TypeReference;
import com.alibaba.nacos.common.utils.CollectionUtils;
import com.alibaba.nacos.common.utils.Objects;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.digiwin.app.container.exceptions.DWBusinessException;
import com.digiwin.app.service.DWEAIResult;
import com.digiwin.athena.app.ptc.infra.entity.LimitManagementDetailEntity;
import com.digiwin.athena.app.ptc.infra.entity.LimitManagementEntity;
import com.digiwin.athena.app.ptc.infra.service.LimitManagementDetailService;
import com.digiwin.athena.app.ptc.infra.service.LimitManagementService;
import com.digiwin.athena.opt.common.eai.EAIRequest;
import com.digiwin.athena.opt.common.eai.service.AbsEAIService;
import com.digiwin.athena.opt.common.security.SecurityUtil;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @auther: zhenggl
 * @date: 2023/9/11
 */
@Service
@Log4j2
public class LimitManagementDetailGetEAIService extends AbsEAIService {


    @Resource
    private LimitManagementDetailService limitManagementDetailService;

    @Resource
    private LimitManagementService limitManagementService;

    @Override
    public String getServiceName() {
        return LimitManagementDetailUtil.LIMIT_MANAGEMENT_DETAIL_GET;
    }

    @Override
    public DWEAIResult execute(Map<String, String> headers, String messageBody) throws Exception {

        //根据id查子表
        EAIRequest eaiRequest = EAIRequest.build(messageBody);
        List<LimitManagementEntity> limitManagementEntities = eaiRequest.getObject("get_data", new TypeReference<List<LimitManagementEntity>>() {});

        if (CollectionUtils.isEmpty(limitManagementEntities)){
            throw new DWBusinessException("必填参数为空");
        }

        LimitManagementEntity limitManagementEntity = limitManagementEntities.get(0);

        Long tenantSid = SecurityUtil.getUserProfile().getTenantSid();
        if (Objects.nonNull(limitManagementEntity.getId())){

            LambdaQueryWrapper<LimitManagementEntity> mfLmq = new LambdaQueryWrapper<>();
            mfLmq.eq(LimitManagementEntity::getId,limitManagementEntity.getId());
            LimitManagementEntity one = limitManagementService.getOne(mfLmq);

            LambdaQueryWrapper<LimitManagementDetailEntity> dfLmq = new LambdaQueryWrapper<>();
            dfLmq.eq(LimitManagementDetailEntity::getTenantSid,tenantSid);
            dfLmq.eq(LimitManagementDetailEntity::getLimitManagementId,limitManagementEntity.getId());
            List<LimitManagementDetailEntity> list = limitManagementDetailService.list(dfLmq);
            one.setInfo(list);
            return buildOK("query_result",one);
        }

        if (Objects.nonNull(limitManagementEntity.getQuotaRatio())){
            //根据间距查询主表
            LambdaQueryWrapper<LimitManagementEntity> lmq = new LambdaQueryWrapper<>();
            lmq.eq(LimitManagementEntity::getTenantSid,tenantSid);
            lmq.le(LimitManagementEntity::getMinPercentage,limitManagementEntity.getQuotaRatio());
            lmq.ge(LimitManagementEntity::getMaxPercentage,limitManagementEntity.getQuotaRatio());
            LimitManagementEntity one = limitManagementService.getOne(lmq);

            //根据id查询子表
            if (Objects.nonNull(one)){
                LambdaQueryWrapper<LimitManagementDetailEntity> dfLmq = new LambdaQueryWrapper<>();
                dfLmq.eq(LimitManagementDetailEntity::getTenantSid,tenantSid);
                dfLmq.eq(LimitManagementDetailEntity::getLimitManagementId,one.getId());
                List<LimitManagementDetailEntity> list = limitManagementDetailService.list(dfLmq);
                one.setInfo(list);
                return buildOK("query_result",one);
            }
        }
        return buildOK("query_result",new HashMap<>());
    }


}
