/*
 * Author:   DONGSK
 * Datetime: 2023/6/30 19:57
 * Description:
 * History:
 * 作者姓名 --修改时间 --版本号--描述
 */
package com.digiwin.athena.app.service.cps;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.digiwin.app.service.DWEAIResult;
import com.digiwin.athena.app.infra.entity.CpsQuestionInfo;
import com.digiwin.athena.app.infra.service.CpsQuestionInfoService;
import com.digiwin.athena.opt.common.eai.EAIRequest;
import com.digiwin.athena.opt.common.eai.EAIUtil;
import com.digiwin.athena.opt.common.eai.service.AbsEAIService;
import com.digiwin.athena.opt.common.security.SecurityUtil;
import com.digiwin.athena.opt.persistence.domain.BaseMgrEntity;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * CpsQuestionUpdateService
 *
 * @author DONGSK 2023/6/30 19:57
 * @since 1.0.0
 */
@Service
public class CpsQuestionGetService extends AbsEAIService {

    @Resource
    CpsQuestionInfoService cpsQuestionInfoService;

    @Override
    public String getServiceName() {
        return CpsUtil.CPS_QUESTION_INFO_GET;
    }

    @Override
    public DWEAIResult execute(Map<String, String> headers, String messageBody) throws Exception {

        EAIRequest eaiRequest = new EAIRequest(messageBody);

        String option = eaiRequest.getString("option");

        Long tenantSid = SecurityUtil.getUserProfile().getTenantSid();

        // 侦测
        if (StringUtils.isBlank(option)) {

            List<CpsQuestionInfo> list = cpsQuestionInfoService.lambdaQuery().eq(CpsQuestionInfo::getStatus, 0).eq(CpsQuestionInfo::getTenantSid,tenantSid).list();
            list.forEach(o -> {
                o.setStatus(1);
            });
            cpsQuestionInfoService.updateBatchById(list);
            return buildOK("change_objects", list);
        }

        // 任务卡查询
        List<Map<String, Object>> questionInfo = eaiRequest.getObject("question_info", EAIUtil.TYPE_LIST_MAP_V_OBJ);

        if (questionInfo.isEmpty()) {
            return buildOK("question_info", new ArrayList<>());
        }

        List<Long> collectIds = questionInfo.stream().map(o -> MapUtils.getLong(o, "id")).collect(Collectors.toList());
        LambdaQueryChainWrapper<CpsQuestionInfo> lambdaQuery = cpsQuestionInfoService.lambdaQuery().eq(CpsQuestionInfo::getTenantSid,tenantSid).in(CpsQuestionInfo::getId, collectIds);

        String status = eaiRequest.getString("status");

        /*switch (option) {
            case "1":
                lambdaQuery.eq(CpsQuestionInfo::getStatus1, MapUtils.getInteger(questionInfo.get(0), "status1"));
                break;
            case "2":
                lambdaQuery.eq(CpsQuestionInfo::getStatus2, MapUtils.getInteger(questionInfo.get(0), "status2"));
                break;
            case "3":
                lambdaQuery.eq(CpsQuestionInfo::getStatus3, MapUtils.getInteger(questionInfo.get(0), "status3"));
                break;
            case "4":
                lambdaQuery.eq(CpsQuestionInfo::getStatus4, MapUtils.getInteger(questionInfo.get(0), "status4"));
                break;
            case "5":
                lambdaQuery.eq(CpsQuestionInfo::getStatus5, MapUtils.getInteger(questionInfo.get(0), "status5"));
                break;
            case "6":
                lambdaQuery.eq(CpsQuestionInfo::getStatus6, MapUtils.getInteger(questionInfo.get(0), "status6"));
                break;
        }*/

        if (StringUtils.isBlank(status)) {
            status = "1";
        }

        switch (option) {
            case "1":
                lambdaQuery.eq(CpsQuestionInfo::getStatus1, status);
                break;
            case "2":
                lambdaQuery.eq(CpsQuestionInfo::getStatus2, status);
                break;
            case "3":
                lambdaQuery.eq(CpsQuestionInfo::getStatus3, status);
                break;
            case "4":
                lambdaQuery.eq(CpsQuestionInfo::getStatus4, status);
                break;
            case "5":
                lambdaQuery.eq(CpsQuestionInfo::getStatus5, status);
                break;
            case "6":
                lambdaQuery.eq(CpsQuestionInfo::getStatus6, status);
                break;
        }

        return buildOK("question_info", lambdaQuery.list());
    }
}
