package com.digiwin.athena.app.service.executor;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.digiwin.app.service.DWEAIResult;
import com.digiwin.athena.app.infra.entity.ItemExecutorEntity;
import com.digiwin.athena.app.infra.repository.ItemExecutorRepository;
import com.digiwin.athena.opt.common.eai.EAIRequest;
import com.digiwin.athena.opt.common.eai.EAIUtil;
import com.digiwin.athena.opt.common.eai.service.AbsEAIService;
import com.digiwin.athena.opt.common.security.SecurityUtil;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @Author: xieps
 * @Date: 2023/8/30 13:55
 * @Version 1.0
 * @Description 品号执行人Get
 */
@Service
@Log4j2
public class ItemExecutorGetEAIService extends AbsEAIService {


    @Resource
    private ItemExecutorRepository itemExecutorRepository;


    @Override
    public String getServiceName() {
        return ExecutorUtil.DEMO_ITEM_EXECUTOR_INFO_GET;
    }

    @Override
    public DWEAIResult execute(Map<String, String> headers, String messageBody) throws Exception {

        EAIRequest request = new EAIRequest(messageBody);
        List<ItemExecutorEntity> itemExecutorInfo = request.getToList("item_executor_info", ItemExecutorEntity.class);
        //组装查询条件  进行查询
        LambdaQueryWrapper<ItemExecutorEntity> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ItemExecutorEntity::getTenantSid, SecurityUtil.getUserProfile().getTenantSid());
        if (!itemExecutorInfo.isEmpty()) {
            queryWrapper.and(
                    queryWrapperInner -> {
                        for (ItemExecutorEntity item : itemExecutorInfo) {
                            queryWrapperInner.or(
                                    wrapper -> wrapper
                                            .eq(!StringUtils.isEmpty(item.getItemNo()), ItemExecutorEntity::getItemNo, item.getItemNo())
                                            .eq(!StringUtils.isEmpty(item.getItemName()), ItemExecutorEntity::getItemName, item.getItemName())
                                            .eq(!StringUtils.isEmpty(item.getProcurementExecutorCode()), ItemExecutorEntity::getProcurementExecutorCode, item.getProcurementExecutorCode())
                                            .eq(!StringUtils.isEmpty(item.getProcurementExecutorName()), ItemExecutorEntity::getProcurementExecutorName, item.getProcurementExecutorName())
                            );
                        }
                    }
            );

        }
        List<ItemExecutorEntity> itemExecutorEntities = itemExecutorRepository.selectList(queryWrapper);
        return EAIUtil.buildEAIResult(new JSONObject().fluentPut("item_executor_info", itemExecutorEntities));
    }
}
