package com.digiwin.athena.app.service.purchase;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.digiwin.app.container.exceptions.DWBusinessException;
import com.digiwin.app.service.DWEAIResult;
import com.digiwin.athena.app.infra.entity.PurchaseOrderDetailEntity;
import com.digiwin.athena.app.infra.repository.PurchaseOrderDetailRepository;
import com.digiwin.athena.app.infra.service.PurchaseDemoService;
import com.digiwin.athena.opt.common.eai.EAIRequest;
import com.digiwin.athena.opt.common.eai.EAIUtil;
import com.digiwin.athena.opt.common.eai.service.AbsEAIService;
import com.digiwin.athena.opt.common.security.SecurityUtil;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author lz
 * @description 创建请购单
 * @throws
 * @time 2023/8/29 16:16
 */
@Service
public class PurchaseDemoCreateService extends AbsEAIService {

    @Resource
    PurchaseOrderDetailRepository purchaseOrderDetailRepository;

    @Resource
    PurchaseDemoService purchaseDemoService;

    @Override
    public String getServiceName() {
        return PurchaseUtil.DEMO_PURCHASE_DEMO_CREATE;
    }

    @Override
    public DWEAIResult execute(Map<String, String> headers, String messageBody) throws Exception {

        EAIRequest request = new EAIRequest(messageBody);
        List<PurchaseOrderDetailEntity> purchaseInfos = request.getObject("purchase_info", PurchaseUtil.LIST_ENTITY_PURCHASE);

        //入参非空校验
        validateParameter(purchaseInfos);

        //集合不为空，插入数据库
        if (!purchaseInfos.isEmpty()) {
            purchaseDemoService.saveBatch(purchaseInfos);
        }

        return EAIUtil.buildEAIResult(new HashMap<>());
    }


    /**
     * @throws
     * @description 校验字段不为空
     * @author lz
     * @time 2023/9/5 15:01
     */
    private void validateParameter(List<PurchaseOrderDetailEntity> purchaseInfos) throws DWBusinessException {
        if (Objects.isNull(purchaseInfos) || purchaseInfos.isEmpty()) {
            throw new DWBusinessException("入参数据不能为空");
        }
        for (PurchaseOrderDetailEntity item : purchaseInfos) {
            if (StringUtils.isEmpty(item.getPurchaseOrderNo())) {
                throw new DWBusinessException("请购单号不能为空或NULL");
            }
            if (StringUtils.isEmpty(item.getPurchaseOrderSeq())) {
                throw new DWBusinessException("请购单序号不能为空或NULL");
            }
            if (StringUtils.isEmpty(item.getItemNo())) {
                throw new DWBusinessException("品号不为空或者NULL");
            }
            if (StringUtils.isEmpty(item.getItemName())) {
                throw new DWBusinessException("品名不为空或者NULL");
            }
            if (StringUtils.isEmpty(item.getProcurementExecutorCode())) {
                throw new DWBusinessException("执行人编号不为空或者NULL");
            }
            if (item.getRequisitionNum() == 0) {
                throw new DWBusinessException("请购数量必须大于0");
            }
        }

        //根据请购单号和请购单序号查询是否存在
        LambdaQueryWrapper<PurchaseOrderDetailEntity> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(PurchaseOrderDetailEntity::getTenantSid, SecurityUtil.getUserProfile().getTenantSid());
        queryWrapper.and(
                queryWrapperInner -> {
                    for (PurchaseOrderDetailEntity purchaseOrderDetail : purchaseInfos) {
                        queryWrapperInner.or(
                                wrapper -> wrapper
                                        .eq(!org.apache.commons.lang.StringUtils.isEmpty(purchaseOrderDetail.getPurchaseOrderNo()), PurchaseOrderDetailEntity::getPurchaseOrderNo, purchaseOrderDetail.getPurchaseOrderNo())
                                        .eq(!org.apache.commons.lang.StringUtils.isEmpty(purchaseOrderDetail.getPurchaseOrderSeq()), PurchaseOrderDetailEntity::getPurchaseOrderSeq, purchaseOrderDetail.getPurchaseOrderSeq())

                        );
                    }
                }
        );
        List<PurchaseOrderDetailEntity> purchaseOrderList = purchaseOrderDetailRepository.selectList(queryWrapper);
        if (purchaseOrderList != null && purchaseOrderList.size() > 0) {
            throw new DWBusinessException("请购单号+请购单序号已存在！");
        }
    }
}
