package com.digiwin.athena.app.service.purchase;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.digiwin.app.service.DWEAIResult;
import com.digiwin.athena.app.infra.common.utils.ResponseUtil;
import com.digiwin.athena.app.infra.common.utils.TransferUtil;
import com.digiwin.athena.app.infra.dto.PurchaseOrderDetailDTO;
import com.digiwin.athena.app.infra.entity.PurchaseOrderDetailEntity;
import com.digiwin.athena.app.infra.repository.PurchaseOrderDetailRepository;
import com.digiwin.athena.opt.common.eai.EAIRequest;
import com.digiwin.athena.opt.common.eai.EAIUtil;
import com.digiwin.athena.opt.common.eai.service.AbsEAIService;
import com.digiwin.athena.opt.common.security.SecurityUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author lizhuangzhuang
 * @description 查询请购单
 * @throws
 * @time 2023/8/30 11:07
 */
@Service
public class PurchaseDemoGetService extends AbsEAIService {

    @Resource
    PurchaseOrderDetailRepository purchaseOrderDetailRepository;

    @Override
    public String getServiceName() {
        return PurchaseUtil.DEMO_PURCHASE_DEMO_GET;
    }

    @Override
    public DWEAIResult execute(Map<String, String> headers, String messageBody) throws Exception {

        EAIRequest request = new EAIRequest(messageBody);
        List<PurchaseOrderDetailEntity> purchaseInfos = request.getObject("purchase_info", PurchaseUtil.LIST_ENTITY_PURCHASE);

        //根据请购单号+请购单序号+状态+任务卡类型查询
        LambdaQueryWrapper<PurchaseOrderDetailEntity> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(PurchaseOrderDetailEntity::getTenantSid, SecurityUtil.getUserProfile().getTenantSid());
        queryWrapper.and(
                queryWrapperInner -> {
                    for (PurchaseOrderDetailEntity purchaseOrderDetail : purchaseInfos) {
                        queryWrapperInner.or(
                                wrapper -> wrapper
                                        .eq(!StringUtils.isEmpty(purchaseOrderDetail.getPurchaseOrderNo()), PurchaseOrderDetailEntity::getPurchaseOrderNo, purchaseOrderDetail.getPurchaseOrderNo())
                                        .eq(!StringUtils.isEmpty(purchaseOrderDetail.getPurchaseOrderSeq()), PurchaseOrderDetailEntity::getPurchaseOrderSeq, purchaseOrderDetail.getPurchaseOrderSeq())
                                        .eq(!StringUtils.isEmpty(purchaseOrderDetail.getTabStatus()), PurchaseOrderDetailEntity::getTabStatus, purchaseOrderDetail.getTabStatus())
                                        .eq(!StringUtils.isEmpty(purchaseOrderDetail.getTaskType()), PurchaseOrderDetailEntity::getTaskType, purchaseOrderDetail.getTaskType())

                        );
                    }
                }
        );
        List<PurchaseOrderDetailEntity> entities = purchaseOrderDetailRepository.selectList(queryWrapper);

        //String类型的null字段，转换为空字符串
        List<PurchaseOrderDetailEntity> purchaseOrderDetailEntities = ResponseUtil.listNullToString(entities);

        List<JSONObject> purchaseInfo = new ArrayList<>();
        for (PurchaseOrderDetailEntity e : purchaseOrderDetailEntities) {
            purchaseInfo.add(JSON.parseObject(new ObjectMapper().writeValueAsString(e)));
        }

        return EAIUtil.buildEAIResult(new JSONObject().fluentPut("purchase_info", purchaseOrderDetailEntities));

    }
}
