package com.digiwin.athena.app.service.purchase;

import com.digiwin.app.container.exceptions.DWBusinessException;
import com.digiwin.app.service.DWEAIResult;
import com.digiwin.athena.app.infra.entity.PurchaseOrderDetailEntity;
import com.digiwin.athena.app.infra.repository.PurchaseOrderDetailRepository;
import com.digiwin.athena.opt.common.eai.EAIRequest;
import com.digiwin.athena.opt.common.eai.EAIUtil;
import com.digiwin.athena.opt.common.eai.service.AbsEAIService;
import com.digiwin.athena.opt.common.security.SecurityUtil;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * @Author: xieps
 * @Date: 2023/9/5 15:29
 * @Version 1.0
 * @Description
 */
@Service
@Log4j2
public class PurchaseOrderUpdateEAIService extends AbsEAIService {


    @Resource
    PurchaseOrderDetailRepository purchaseOrderDetailRepository;

    @Override
    public String getServiceName() {
        return PurchaseUtil.DEMO_PURCHASE_ORDER_UPDATE;
    }

    @Override
    public DWEAIResult execute(Map<String, String> headers, String messageBody) throws Exception {

        EAIRequest request = new EAIRequest(messageBody);
        List<PurchaseOrderDetailEntity> entityList = request.getToList("purchase_info", PurchaseOrderDetailEntity.class);
        //校验请购单号、序号 必传
        validateParameter(entityList);
        //管理字段赋值
        entityList.stream().forEach(item -> {
            item.setTenantSid(SecurityUtil.getUserProfile().getTenantSid());
            item.setModifiedBy(SecurityUtil.getUserProfile().getUserId());
            item.setModifiedDate(new Date());
        });
        //根据单号，序号进行批量更新
        purchaseOrderDetailRepository.updateBatch(entityList);

        return EAIUtil.buildEAIResult(new HashMap<>());
    }


    /**
     * 校验必传参数
     *
     * @param entityList
     */
    private void validateParameter(List<PurchaseOrderDetailEntity> entityList) throws DWBusinessException {
        if (Objects.isNull(entityList) || entityList.isEmpty()) {
            throw new DWBusinessException("入参数据为空~~");
        }
        for (PurchaseOrderDetailEntity entity : entityList) {

            if (StringUtils.isEmpty(entity.getPurchaseOrderNo())) {
                throw new DWBusinessException("请购单号不为空或者NULL~");
            }

            if (StringUtils.isEmpty(entity.getPurchaseOrderSeq())) {
                throw new DWBusinessException("请购单序号不为空或者NULL~");
            }
        }
    }


}
