package com.digiwin.athena.app.service.staff;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.digiwin.app.container.exceptions.DWRuntimeException;
import com.digiwin.app.service.DWEAIResult;
import com.digiwin.athena.app.infra.common.enums.OptionEnums;
import com.digiwin.athena.app.infra.common.enums.StatusEnums;
import com.digiwin.athena.app.infra.common.enums.TabStatusEnums;
import com.digiwin.athena.app.infra.entity.StaffEntity;
import com.digiwin.athena.app.infra.repository.StaffRepository;
import com.digiwin.athena.app.infra.service.StaffService;
import com.digiwin.athena.opt.common.eai.EAIRequest;
import com.digiwin.athena.opt.common.eai.EAIUtil;
import com.digiwin.athena.opt.common.eai.service.AbsEAIService;
import com.digiwin.athena.opt.common.security.SecurityUtil;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * @Auther: zhenggl
 * @Date: 2023/4/28
 */
@Service
@Log4j2
public class StaffUpdateEAIService extends AbsEAIService {

    @Resource
    private StaffRepository staffRepository;

    @Resource
    private StaffService staffService;

    @Override
    public String getServiceName() {
        return StaffUtil.DEMO_WAIT_REGISTER_PERSONNEL_INFO_UPDATE;
    }

    @Override
    public DWEAIResult execute(Map<String, String> headers, String messageBody) throws Exception {
        //反序列化字段
        EAIRequest request = new EAIRequest(messageBody);
        String option = request.getObject("option", String.class);
        List<StaffEntity> staffList = request.getObject("wait_register_personnel", StaffUtil.LIST_ENTITY_STAFF);
        if (CollectionUtils.isEmpty(staffList)) {
            throw new DWRuntimeException("必要参数不可为空");
        }
        //界面
        if (StringUtils.isEmpty(option)) {
                staffList.forEach(item ->{
                    item.setStatus(StatusEnums.TOBEREPORTED.getValue());
                    item.setTabStatus(TabStatusEnums.NOSET.getValue());
                });
                staffService.saveBatch(staffList);
        } else if (OptionEnums.BASIC.getValue().equals(option)) {
                //区分是否有id没有则新增
                Map<Boolean, List<StaffEntity>> collect = staffList.stream().collect(Collectors.groupingBy(item -> ObjectUtils.isEmpty(item.getId())));
                List<StaffEntity> insert = collect.get(true);
                if (!CollectionUtils.isEmpty(insert)){
                    insert.forEach(item ->{
                        item.setStatus(StatusEnums.TOBEREPORTED.getValue());
                        item.setTabStatus(TabStatusEnums.NOSET.getValue());
                    });
                    staffService.saveBatch(insert);
                }
                List<StaffEntity> update = collect.get(false);
                if (!CollectionUtils.isEmpty(update)){
                    staffService.updateBatchById(update);
                }
        } else if (OptionEnums.TASK.getValue().equals(option)) {
                staffRepository.updateStatusAndDate(staffList, SecurityUtil.getUserProfile().getTenantSid());
        } else if (OptionEnums.FLOW.getValue().equals(option)) {
            List<Long> idList = staffList.stream().map(StaffEntity::getId).collect(Collectors.toList());
            LambdaUpdateWrapper<StaffEntity> lmu = new LambdaUpdateWrapper<>();
            lmu.set(StaffEntity::getStatus, StatusEnums.REPORTED.getValue());
            lmu.in(StaffEntity::getId, idList);
            staffService.update(lmu);
        }
        return EAIUtil.buildEAIResult(new HashMap<>());
    }
}
