package com.digiwin.athena.app.service.supplier;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.nacos.common.utils.CollectionUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.digiwin.app.service.DWEAIResult;
import com.digiwin.athena.app.infra.entity.FontSupplierEntity;
import com.digiwin.athena.app.infra.service.FontSupplierService;
import com.digiwin.athena.opt.common.eai.EAIRequest;
import com.digiwin.athena.opt.common.eai.EAIUtil;
import com.digiwin.athena.opt.common.eai.service.AbsEAIService;
import com.digiwin.athena.opt.common.security.SecurityUtil;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * @auther: bk
 * @date: 2023/8/31
 */
@Service
@Log4j2
public class FontSupplierGetEAIService extends AbsEAIService {


    @Resource
    private FontSupplierService fontSupplierService;

    @Override
    public String getServiceName() {
        return FontSupplierUtil.DEMO_FONT_ITEM_SUPPLIER_INFO_GET;
    }

    @Override
    public DWEAIResult execute(Map<String, String> headers, String messageBody) throws Exception {
        EAIRequest eaiRequest = EAIRequest.build(messageBody);
        List<FontSupplierEntity> supplierInfo = eaiRequest.getObject("item_supplier_info", FontSupplierUtil.LIST_ENTITY_SUPPLIER);
        LambdaQueryWrapper<FontSupplierEntity> lmq = new LambdaQueryWrapper<>();
        lmq.eq(FontSupplierEntity::getTenantId, SecurityUtil.getUserProfile().getTenantId());
        if (CollectionUtils.isNotEmpty(supplierInfo)) {
            lmq.and(
                    queryWrapperInner -> {
                        for (FontSupplierEntity fontSupplierEntity : supplierInfo) {
                            queryWrapperInner.or(
                                    wrapper -> wrapper
                                            .eq(FontSupplierEntity::getItemNo, fontSupplierEntity.getItemNo())
                                            .eq(FontSupplierEntity::getItemName, fontSupplierEntity.getItemName())
                                            .eq(FontSupplierEntity::getSupplierNo, fontSupplierEntity.getSupplierNo())
                                            .eq(FontSupplierEntity::getSupplierName, fontSupplierEntity.getSupplierName())
                            );
                        }
                    });
        }

        List<FontSupplierEntity> list = fontSupplierService.list(lmq);
        return EAIUtil.buildEAIResult(new JSONObject().fluentPut("item_supplier_info", list));
    }
}
