package com.digiwin.athena.app.service.supplier;

import com.alibaba.fastjson.JSON;
import com.digiwin.app.container.exceptions.DWBusinessException;
import com.digiwin.app.service.DWEAIResult;
import com.digiwin.athena.app.infra.entity.FontSupplierEntity;
import com.digiwin.athena.app.infra.entity.ItemExecutorEntity;
import com.digiwin.athena.app.infra.service.FontSupplierService;
import com.digiwin.athena.opt.common.eai.EAIRequest;
import com.digiwin.athena.opt.common.eai.EAIUtil;
import com.digiwin.athena.opt.common.eai.service.AbsEAIService;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentSkipListMap;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @auther: bk
 * @date: 2023/8/31
 */
@Service
@Log4j2
public class FontSupplierUpdateEAIService extends AbsEAIService {

    @Resource
    private FontSupplierService supplierService;

    @Override
    public String getServiceName() {
        return FontSupplierUtil.DEMO_FONT_ITEM_SUPPLIER_INFO_UPDATE;
    }

    @Override
    public DWEAIResult execute(Map<String, String> headers, String messageBody) throws Exception {
        EAIRequest eaiRequest = EAIRequest.build(messageBody);
        List<FontSupplierEntity> supplierInfo = eaiRequest.getObject("item_supplier_info", FontSupplierUtil.LIST_ENTITY_SUPPLIER);

        //对必传参数 且 品号和执行人编号不可以重复 进行校验
        validateParameter(supplierInfo);

        supplierService.updateBatchById(supplierInfo);
        return EAIUtil.buildEAIResult(new HashMap<>());
    }

    private void validateParameter(List<FontSupplierEntity> supplierInfo)  throws DWBusinessException {
        if (Objects.isNull(supplierInfo) || supplierInfo.isEmpty()) {
            throw new DWBusinessException("入参数据为空");
        }
        for (FontSupplierEntity item : supplierInfo) {
            if (StringUtils.isEmpty(item.getItemNo())) {
                throw new DWBusinessException("品号字段值为空或者NULL");
            }
            if (StringUtils.isEmpty(item.getItemName())) {
                throw new DWBusinessException("品名字段值为空或者NULL");
            }
            if (StringUtils.isEmpty(item.getSupplierNo())) {
                throw new DWBusinessException("供应商no为空或者NULL");
            }
            if (StringUtils.isEmpty(item.getSupplierName())) {
                throw new DWBusinessException("供应商name为空或者NULL");
            }
        }
        List<FontSupplierEntity> distinctList = supplierInfo.stream().filter(distinctByKey(item -> Stream.of(item.getItemNo(), item.getSupplierNo()).toArray())).collect(Collectors.toList());
        if (supplierInfo.size() > distinctList.size()) {
            throw new DWBusinessException("品号和供应商编号入参重复");
        }
    }

    /**
     * 用于对象去重
     *
     * @param keyExtractor 需要去重的属性
     * @param <T>
     * @return
     */
    private static <T> Predicate<T> distinctByKey(Function<? super T, ?> keyExtractor) {
        //记录已有对象或者属性
        ConcurrentSkipListMap<Object, Boolean> skipListMap = new ConcurrentSkipListMap<>();
        return t -> skipListMap.putIfAbsent(JSON.toJSONString(keyExtractor.apply(t)), Boolean.TRUE) == null;
    }
}
