package com.digiwin.athena.app.service.supplier;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.digiwin.app.service.DWEAIResult;
import com.digiwin.athena.app.infra.entity.SupplierContactEntity;
import com.digiwin.athena.app.infra.service.SupplierContactInfoService;
import com.digiwin.athena.opt.common.eai.EAIRequest;
import com.digiwin.athena.opt.common.eai.EAIUtil;
import com.digiwin.athena.opt.common.eai.service.AbsEAIService;
import com.digiwin.athena.opt.common.generator.SnowflakeWorker;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author CR-7
 * create: 2023-09-01 16:03
 * Description:
 */
@Log4j2
@Service
public class SupplierContactInfoCreateEAIService extends AbsEAIService {


    @Resource
    SupplierContactInfoService supplierContactInfoService;

    @Override
    public String getServiceName() {
        return SupplierContactUtil.DEMO_DATA_CONTACT_INFO_CREATE;
    }

    @Override
    public DWEAIResult execute(Map<String, String> headers, String messageBody) throws Exception {
        // 入参反序列化
        EAIRequest request = new EAIRequest(messageBody);
        List<SupplierContactEntity> supplierContactEntityList = request.getObject("contact_info", new TypeReference<List<SupplierContactEntity>>(){});

        List<SupplierContactEntity> supplierContactEntities = supplierContactEntityList.stream().map(assemble->{
            SupplierContactEntity supplierContactEntity = new SupplierContactEntity();
            supplierContactEntity.setId(SnowflakeWorker.nextId());
            BeanUtils.copyProperties(assemble,supplierContactEntity);
            return supplierContactEntity;
        }).collect(Collectors.toList());

        this.supplierContactInfoService.saveBatch(supplierContactEntities);

        return EAIUtil.buildEAIResult(new JSONObject().fluentPut("contact_info", supplierContactEntities));
    }
}
