package com.digiwin.athena.app.service.supplier;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.nacos.common.utils.CollectionUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.digiwin.app.service.DWEAIResult;
import com.digiwin.athena.app.infra.entity.ItemSupplierEntity;
import com.digiwin.athena.app.infra.service.SupplierService;
import com.digiwin.athena.opt.common.eai.EAIRequest;
import com.digiwin.athena.opt.common.eai.EAIUtil;
import com.digiwin.athena.opt.common.eai.service.AbsEAIService;
import com.digiwin.athena.opt.common.security.SecurityUtil;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * @auther: zhenggl
 * @date: 2023/8/31
 */
@Service
@Log4j2
public class SupplierGetEAIService extends AbsEAIService {


    @Resource
    private SupplierService supplierService;

    @Override
    public String getServiceName() {
        return SupplierUtil.DEMO_ITEM_SUPPLIER_INFO_GET;
    }

    @Override
    public DWEAIResult execute(Map<String, String> headers, String messageBody) throws Exception {
        EAIRequest eaiRequest = EAIRequest.build(messageBody);
        List<ItemSupplierEntity> supplierInfo = eaiRequest.getObject("item_supplier_info", SupplierUtil.LIST_ENTITY_SUPPLIER);
        LambdaQueryWrapper<ItemSupplierEntity> lmq = new LambdaQueryWrapper<>();
        lmq.eq(ItemSupplierEntity::getTenantId, SecurityUtil.getUserProfile().getTenantId());
        if (CollectionUtils.isNotEmpty(supplierInfo)) {
            lmq.and(
                    queryWrapperInner -> {
                        for (ItemSupplierEntity itemSupplierEntity : supplierInfo) {
                            queryWrapperInner.or(
                                    wrapper -> wrapper
                                            .eq(ItemSupplierEntity::getItemNo, itemSupplierEntity.getItemNo())
                                            .eq(ItemSupplierEntity::getItemName, itemSupplierEntity.getItemName())
                                            .eq(ItemSupplierEntity::getSupplierNo, itemSupplierEntity.getSupplierNo())
                                            .eq(ItemSupplierEntity::getSupplierName, itemSupplierEntity.getSupplierName())
                            );
                        }
                    });
        }

        List<ItemSupplierEntity> list = supplierService.list(lmq);
        return EAIUtil.buildEAIResult(new JSONObject().fluentPut("item_supplier_info", list));
    }
}
